@@ -606,8 +606,44 @@ action_global_subscribe_foreign_folder_cb (GtkAction *action,
 	g_object_unref (session);
 }
 
-static GtkActionEntry global_mapi_entries[] = {
-	{ "mapi-global-subscribe-foreign-folder",
+static GtkActionEntry global_mapi_mail_entries[] = {
+	{ "mapi-mail-global-subscribe-foreign-folder",
+	  NULL,
+	  N_("Subscribe to folder of other MAPI user…"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  G_CALLBACK (action_global_subscribe_foreign_folder_cb) }
+};
+
+static GtkActionEntry global_mapi_book_entries[] = {
+	{ "mapi-contact-global-subscribe-foreign-folder",
+	  NULL,
+	  N_("Subscribe to folder of other MAPI user…"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  G_CALLBACK (action_global_subscribe_foreign_folder_cb) }
+};
+
+static GtkActionEntry global_mapi_cal_entries[] = {
+	{ "mapi-calendar-global-subscribe-foreign-folder",
+	  NULL,
+	  N_("Subscribe to folder of other MAPI user…"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  G_CALLBACK (action_global_subscribe_foreign_folder_cb) }
+};
+
+static GtkActionEntry global_mapi_task_entries[] = {
+	{ "mapi-task-global-subscribe-foreign-folder",
+	  NULL,
+	  N_("Subscribe to folder of other MAPI user…"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  G_CALLBACK (action_global_subscribe_foreign_folder_cb) }
+};
+
+static GtkActionEntry global_mapi_memo_entries[] = {
+	{ "mapi-memo-global-subscribe-foreign-folder",
 	  NULL,
 	  N_("Subscribe to folder of other MAPI user…"),
 	  NULL,
@@ -896,7 +932,7 @@ static const gchar *mapi_ui_mail_def =
 	"<menubar name='main-menu'>\n"
 	"  <menu action='file-menu'>\n"
 	"    <placeholder name='long-running-actions'>\n"
-	"      <menuitem action=\"mapi-global-subscribe-foreign-folder\"/>\n"
+	"      <menuitem action=\"mapi-mail-global-subscribe-foreign-folder\"/>\n"
 	"    </placeholder>\n"
 	"  </menu>\n"
 	"</menubar>\n"
@@ -959,7 +995,7 @@ mapi_ui_update_actions_mail_cb (EShellView *shell_view,
 
 	mapi_ui_enable_actions (action_group, mail_account_context_entries, G_N_ELEMENTS (mail_account_context_entries), account_node, online);
 	mapi_ui_enable_actions (action_group, mail_folder_context_entries, G_N_ELEMENTS (mail_folder_context_entries), folder_node, online);
-	mapi_ui_enable_actions (action_group, global_mapi_entries, G_N_ELEMENTS (global_mapi_entries), has_mapi_account, online);
+	mapi_ui_enable_actions (action_group, global_mapi_mail_entries, G_N_ELEMENTS (global_mapi_mail_entries), has_mapi_account, online);
 }
 
 static void
@@ -986,7 +1022,7 @@ mapi_ui_init_mail (GtkUIManager *ui_manager,
 	/* Add global actions */
 	e_action_group_add_actions_localized (
 		action_group, GETTEXT_PACKAGE,
-		global_mapi_entries, G_N_ELEMENTS (global_mapi_entries), shell_view);
+		global_mapi_mail_entries, G_N_ELEMENTS (global_mapi_mail_entries), shell_view);
 
 	/* Decide whether we want this option to be visible or not */
 	g_signal_connect (shell_view, "update-actions",
@@ -1058,22 +1094,33 @@ update_mapi_source_entries_cb (EShellView *shell_view,
 	EShell *shell;
 	EShellWindow *shell_window;
 	ESource *source = NULL;
+	GtkActionEntry *global_entries = NULL;
+	guint n_global_entries = 0;
 	const gchar *group;
 	gboolean is_mapi_source, is_online;
 
 	g_return_if_fail (E_IS_SHELL_VIEW (shell_view));
 	g_return_if_fail (entries != NULL);
 
-	if (strstr (entries->name, "calendar"))
+	if (strstr (entries->name, "calendar")) {
 		group = "calendar";
-	else if (strstr (entries->name, "tasks"))
+		global_entries = global_mapi_cal_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_cal_entries);
+	} else if (strstr (entries->name, "tasks")) {
 		group = "tasks";
-	else if (strstr (entries->name, "memos"))
+		global_entries = global_mapi_task_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_task_entries);
+	} else if (strstr (entries->name, "memos")) {
 		group = "memos";
-	else if (strstr (entries->name, "contacts"))
+		global_entries = global_mapi_memo_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_memo_entries);
+	} else if (strstr (entries->name, "contacts")) {
 		group = "contacts";
-	else
+		global_entries = global_mapi_book_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_book_entries);
+	} else {
 		g_return_if_reached ();
+	}
 
 	is_mapi_source = get_selected_mapi_source (shell_view, &source, NULL);
 
@@ -1097,7 +1144,7 @@ update_mapi_source_entries_cb (EShellView *shell_view,
 	action_group = e_shell_window_get_action_group (shell_window, group);
 
 	mapi_ui_enable_actions (action_group, entries, MAPI_ESOURCE_NUM_ENTRIES, is_mapi_source, is_online);
-	mapi_ui_enable_actions (action_group, global_mapi_entries, G_N_ELEMENTS (global_mapi_entries),
+	mapi_ui_enable_actions (action_group, global_entries, n_global_entries,
 		mapi_ui_has_mapi_account (shell_view, NULL), is_online);
 }
 
@@ -1109,6 +1156,8 @@ setup_mapi_source_actions (EShellView *shell_view,
 {
 	EShellWindow *shell_window;
 	GtkActionGroup *action_group;
+	GtkActionEntry *global_entries = NULL;
+	guint n_global_entries = 0;
 	const gchar *group;
 
 	g_return_if_fail (shell_view != NULL);
@@ -1117,16 +1166,25 @@ setup_mapi_source_actions (EShellView *shell_view,
 	g_return_if_fail (n_entries > 0);
 	g_return_if_fail (n_entries == MAPI_ESOURCE_NUM_ENTRIES);
 
-	if (strstr (entries->name, "calendar"))
+	if (strstr (entries->name, "calendar")) {
 		group = "calendar";
-	else if (strstr (entries->name, "tasks"))
+		global_entries = global_mapi_cal_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_cal_entries);
+	} else if (strstr (entries->name, "tasks")) {
 		group = "tasks";
-	else if (strstr (entries->name, "memos"))
+		global_entries = global_mapi_task_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_task_entries);
+	} else if (strstr (entries->name, "memos")) {
 		group = "memos";
-	else if (strstr (entries->name, "contacts"))
+		global_entries = global_mapi_memo_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_memo_entries);
+	} else if (strstr (entries->name, "contacts")) {
 		group = "contacts";
-	else
+		global_entries = global_mapi_book_entries;
+		n_global_entries = G_N_ELEMENTS (global_mapi_book_entries);
+	} else {
 		g_return_if_reached ();
+	}
 
 	shell_window = e_shell_view_get_shell_window (shell_view);
 	action_group = e_shell_window_get_action_group (shell_window, group);
@@ -1138,7 +1196,7 @@ setup_mapi_source_actions (EShellView *shell_view,
 	/* Add global actions */
 	e_action_group_add_actions_localized (
 		action_group, GETTEXT_PACKAGE,
-		global_mapi_entries, G_N_ELEMENTS (global_mapi_entries), shell_view);
+		global_entries, n_global_entries, shell_view);
 
 	g_signal_connect (shell_view, "update-actions", G_CALLBACK (update_mapi_source_entries_cb), entries);
 }
@@ -1210,7 +1268,7 @@ static const gchar *mapi_ui_cal_def =
 	"<menubar name='main-menu'>\n"
 	"  <menu action='file-menu'>\n"
 	"    <placeholder name='long-running-actions'>\n"
-	"      <menuitem action=\"mapi-global-subscribe-foreign-folder\"/>\n"
+	"      <menuitem action=\"mapi-calendar-global-subscribe-foreign-folder\"/>\n"
 	"    </placeholder>\n"
 	"  </menu>\n"
 	"</menubar>\n"
@@ -1247,7 +1305,7 @@ static const gchar *mapi_ui_task_def =
 	"<menubar name='main-menu'>\n"
 	"  <menu action='file-menu'>\n"
 	"    <placeholder name='long-running-actions'>\n"
-	"      <menuitem action=\"mapi-global-subscribe-foreign-folder\"/>\n"
+	"      <menuitem action=\"mapi-task-global-subscribe-foreign-folder\"/>\n"
 	"    </placeholder>\n"
 	"  </menu>\n"
 	"</menubar>\n"
@@ -1284,7 +1342,7 @@ static const gchar *mapi_ui_memo_def =
 	"<menubar name='main-menu'>\n"
 	"  <menu action='file-menu'>\n"
 	"    <placeholder name='long-running-actions'>\n"
-	"      <menuitem action=\"mapi-global-subscribe-foreign-folder\"/>\n"
+	"      <menuitem action=\"mapi-memo-global-subscribe-foreign-folder\"/>\n"
 	"    </placeholder>\n"
 	"  </menu>\n"
 	"</menubar>\n"
@@ -1321,7 +1379,7 @@ static const gchar *mapi_ui_book_def =
 	"<menubar name='main-menu'>\n"
 	"  <menu action='file-menu'>\n"
 	"    <placeholder name='long-running-actions'>\n"
-	"      <menuitem action=\"mapi-global-subscribe-foreign-folder\"/>\n"
+	"      <menuitem action=\"mapi-contact-global-subscribe-foreign-folder\"/>\n"
 	"    </placeholder>\n"
 	"  </menu>\n"
 	"</menubar>\n"
